const fs = require('fs');
const { createCanvas, registerFont, deregisterAllFonts } = require('canvas');

const defaultFontFamily = '"Open Sans"';

const registerFonts = () => {
    registerFont("./fonts/OpenSans-Light.ttf", {
        family: "Open Sans",
        weight: "300",
    });
    registerFont("./fonts/OpenSans-Regular.ttf", {
        family: "Open Sans",
        weight: "400",
    });
    registerFont("./fonts/OpenSans-Bold.ttf", {
        family: "Open Sans",
        weight: "700",
    });
    registerFont("./fonts/OpenSans-ExtraBold.ttf", {
        family: "Open Sans",
        weight: "800",
    });
};

const generateImage = async () => {
    const canvas = createCanvas(480, 240);
    const ctx = canvas.getContext("2d");
    ctx.quality = "best";

    ctx.fillStyle = "#663399";
    ctx.fillRect(0, 0, canvas.width, canvas.height);

    ctx.fillStyle = "#fff";
    ctx.font = `400 40px ${defaultFontFamily}`;
    ctx.textBaseline = "top";
    ctx.fillText("REGULAR", 40, canvas.height / 2 - 50);

    ctx.font = `800 40px ${defaultFontFamily}`;
    ctx.fillText("EXTRABOLD", 40, canvas.height / 2);

    // Generate PNG output stream
    return canvas.createPNGStream();
};

const handler = async () => {
    registerFonts();

    const outputStream = await generateImage();
    const file = fs.createWriteStream("canvas.png");

    return new Promise((resolve, reject) => {
        outputStream.pipe(file);
        outputStream.on("finish", () => {
            deregisterAllFonts();
            resolve();
        });
        outputStream.on("error", (err) => reject(err.message));
    });
};

handler();
